/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef GET_FILE_H
#define GET_FILE_H

#include "AutomateCommand.h"
#include "DiffParser.h"

#include <QAbstractItemModel>

// FIXME: theoretically we could now just use the DiffLine struct which has
// the same fields, however a content line might not always keep a diff line
// forever...
struct ContentLine {
    QString content;
    DiffLine::State state;
    int leftLineCount;
    int rightLineCount;
    ContentLine() {}
    ContentLine(QString c, DiffLine::State s, int left, int right)
        : content(c), state(s), leftLineCount(left), rightLineCount(right) {}
};

typedef QVector<ContentLine *> Content;

class GetFile : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    GetFile(QObject *, const DatabaseFile &);
    virtual ~GetFile();

    // needed Qt Model methods
    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;
    QModelIndex getNextGroup(const QModelIndex &, bool recurse = false);
    QModelIndex getPrevGroup(const QModelIndex &, bool recurse = false);

public slots:
    void readFileById(const QString &);
    void readFileByName(const QString &, const QString &);
    void applyDiff(Diff * diff);

signals:
    void fileRead();

private:
    void readFile(QStringList, QStringList);
    void processTaskResult(const MonotoneTask &);

    Content fileContents;
    DatabaseFile databaseFile;
};

#endif

